/*   GENERATED FILE DO NOT EDIT
*
*  === This file is part of Calamares - <https://calamares.io> ===
*
* SPDX-FileCopyrightText: 1991-2019 Unicode, Inc.
* SPDX-FileCopyrightText: 2019 Adriaan de Groot <groot@kde.org>
* SPDX-License-Identifier: CC0-1.0
*
* This file is derived from CLDR data from Unicode, Inc. Applicable terms
* are listed at http://unicode.org/copyright.html , of which the most
* important are:
*
* A. Unicode Copyright
*    1. Copyright © 1991-2019 Unicode, Inc. All rights reserved.
* B. Definitions
*    Unicode Data Files ("DATA FILES") include all data files under the directories:
*    https://www.unicode.org/Public/
* C. Terms of Use
*    1. Certain documents and files on this website contain a legend indicating
*       that "Modification is permitted." Any person is hereby authorized,
*       without fee, to modify such documents and files to create derivative
*       works conforming to the Unicode® Standard, subject to Terms and
*       Conditions herein.
*    2. Any person is hereby authorized, without fee, to view, use, reproduce,
*       and distribute all documents and files, subject to the Terms and
*       Conditions herein.
*/

/*  MODIFICATIONS
 *
 * Edited anyway:
 *      20191211 India (IN) changed to AnyLanguage, since Hindi doesn't make sense. #1284
 *      20210207 Belarus (BY) changed to Russian, as the more-common-language. #1634
 *      20210615 Tokelau and Tuvalu country enum values changed to avoid deprecation warning.
 *
 */

// BEGIN Generated from CLDR data
// *INDENT-OFF*
// clang-format off


struct CountryData
{
    QLocale::Language l;
    QLocale::Country c;
    char cc1;
    char cc2;
};

static constexpr int const country_data_size = 198;

static const CountryData country_data_table[] = {
{ QLocale::Language::Catalan, QLocale::Country::Andorra, 'A', 'D' },
{ QLocale::Language::Arabic, QLocale::Country::UnitedArabEmirates, 'A', 'E' },
{ QLocale::Language::Persian, QLocale::Country::Afghanistan, 'A', 'F' },
{ QLocale::Language::Albanian, QLocale::Country::Albania, 'A', 'L' },
{ QLocale::Language::Armenian, QLocale::Country::Armenia, 'A', 'M' },
{ QLocale::Language::Portuguese, QLocale::Country::Angola, 'A', 'O' },
{ QLocale::Language::AnyLanguage, QLocale::Country::Antarctica, 'A', 'Q' },
{ QLocale::Language::Spanish, QLocale::Country::Argentina, 'A', 'R' },
{ QLocale::Language::Samoan, QLocale::Country::AmericanSamoa, 'A', 'S' },
{ QLocale::Language::German, QLocale::Country::Austria, 'A', 'T' },
{ QLocale::Language::Dutch, QLocale::Country::Aruba, 'A', 'W' },
{ QLocale::Language::Swedish, QLocale::Country::AlandIslands, 'A', 'X' },
{ QLocale::Language::Azerbaijani, QLocale::Country::Azerbaijan, 'A', 'Z' },
{ QLocale::Language::Bosnian, QLocale::Country::BosniaAndHerzegowina, 'B', 'A' },
{ QLocale::Language::Bengali, QLocale::Country::Bangladesh, 'B', 'D' },
{ QLocale::Language::Dutch, QLocale::Country::Belgium, 'B', 'E' },
{ QLocale::Language::French, QLocale::Country::BurkinaFaso, 'B', 'F' },
{ QLocale::Language::Bulgarian, QLocale::Country::Bulgaria, 'B', 'G' },
{ QLocale::Language::Arabic, QLocale::Country::Bahrain, 'B', 'H' },
{ QLocale::Language::Rundi, QLocale::Country::Burundi, 'B', 'I' },
{ QLocale::Language::French, QLocale::Country::Benin, 'B', 'J' },
{ QLocale::Language::French, QLocale::Country::SaintBarthelemy, 'B', 'L' },
{ QLocale::Language::Malay, QLocale::Country::Brunei, 'B', 'N' },
{ QLocale::Language::Spanish, QLocale::Country::Bolivia, 'B', 'O' },
{ QLocale::Language::Papiamento, QLocale::Country::Bonaire, 'B', 'Q' },
{ QLocale::Language::Portuguese, QLocale::Country::Brazil, 'B', 'R' },
{ QLocale::Language::Dzongkha, QLocale::Country::Bhutan, 'B', 'T' },
{ QLocale::Language::AnyLanguage, QLocale::Country::BouvetIsland, 'B', 'V' },
{ QLocale::Language::Russian, QLocale::Country::Belarus, 'B', 'Y' },
{ QLocale::Language::Swahili, QLocale::Country::CongoKinshasa, 'C', 'D' },
{ QLocale::Language::French, QLocale::Country::CentralAfricanRepublic, 'C', 'F' },
{ QLocale::Language::French, QLocale::Country::CongoBrazzaville, 'C', 'G' },
{ QLocale::Language::German, QLocale::Country::Switzerland, 'C', 'H' },
{ QLocale::Language::French, QLocale::Country::IvoryCoast, 'C', 'I' },
{ QLocale::Language::Spanish, QLocale::Country::Chile, 'C', 'L' },
{ QLocale::Language::French, QLocale::Country::Cameroon, 'C', 'M' },
{ QLocale::Language::Chinese, QLocale::Country::China, 'C', 'N' },
{ QLocale::Language::Spanish, QLocale::Country::Colombia, 'C', 'O' },
{ QLocale::Language::AnyLanguage, QLocale::Country::ClippertonIsland, 'C', 'P' },
{ QLocale::Language::Spanish, QLocale::Country::CostaRica, 'C', 'R' },
{ QLocale::Language::Spanish, QLocale::Country::Cuba, 'C', 'U' },
{ QLocale::Language::Portuguese, QLocale::Country::CapeVerde, 'C', 'V' },
{ QLocale::Language::Papiamento, QLocale::Country::CuraSao, 'C', 'W' },
{ QLocale::Language::Greek, QLocale::Country::Cyprus, 'C', 'Y' },
{ QLocale::Language::Czech, QLocale::Country::CzechRepublic, 'C', 'Z' },
{ QLocale::Language::German, QLocale::Country::Germany, 'D', 'E' },
{ QLocale::Language::Afar, QLocale::Country::Djibouti, 'D', 'J' },
{ QLocale::Language::Danish, QLocale::Country::Denmark, 'D', 'K' },
{ QLocale::Language::Spanish, QLocale::Country::DominicanRepublic, 'D', 'O' },
{ QLocale::Language::Arabic, QLocale::Country::Algeria, 'D', 'Z' },
{ QLocale::Language::Spanish, QLocale::Country::CeutaAndMelilla, 'E', 'A' },
{ QLocale::Language::Spanish, QLocale::Country::Ecuador, 'E', 'C' },
{ QLocale::Language::Estonian, QLocale::Country::Estonia, 'E', 'E' },
{ QLocale::Language::Arabic, QLocale::Country::Egypt, 'E', 'G' },
{ QLocale::Language::Arabic, QLocale::Country::WesternSahara, 'E', 'H' },
{ QLocale::Language::Tigrinya, QLocale::Country::Eritrea, 'E', 'R' },
{ QLocale::Language::Spanish, QLocale::Country::Spain, 'E', 'S' },
{ QLocale::Language::Amharic, QLocale::Country::Ethiopia, 'E', 'T' },
{ QLocale::Language::English, QLocale::Country::EuropeanUnion, 'E', 'U' },
{ QLocale::Language::German, QLocale::Country::AnyCountry, 'E', 'Z' },
{ QLocale::Language::Finnish, QLocale::Country::Finland, 'F', 'I' },
{ QLocale::Language::Faroese, QLocale::Country::FaroeIslands, 'F', 'O' },
{ QLocale::Language::French, QLocale::Country::France, 'F', 'R' },
{ QLocale::Language::French, QLocale::Country::Gabon, 'G', 'A' },
{ QLocale::Language::Georgian, QLocale::Country::Georgia, 'G', 'E' },
{ QLocale::Language::French, QLocale::Country::FrenchGuiana, 'G', 'F' },
{ QLocale::Language::Akan, QLocale::Country::Ghana, 'G', 'H' },
{ QLocale::Language::Greenlandic, QLocale::Country::Greenland, 'G', 'L' },
{ QLocale::Language::French, QLocale::Country::Guinea, 'G', 'N' },
{ QLocale::Language::French, QLocale::Country::Guadeloupe, 'G', 'P' },
{ QLocale::Language::Spanish, QLocale::Country::EquatorialGuinea, 'G', 'Q' },
{ QLocale::Language::Greek, QLocale::Country::Greece, 'G', 'R' },
{ QLocale::Language::AnyLanguage, QLocale::Country::SouthGeorgiaAndTheSouthSandwichIslands, 'G', 'S' },
{ QLocale::Language::Spanish, QLocale::Country::Guatemala, 'G', 'T' },
{ QLocale::Language::Portuguese, QLocale::Country::GuineaBissau, 'G', 'W' },
{ QLocale::Language::Chinese, QLocale::Country::HongKong, 'H', 'K' },
{ QLocale::Language::AnyLanguage, QLocale::Country::HeardAndMcDonaldIslands, 'H', 'M' },
{ QLocale::Language::Spanish, QLocale::Country::Honduras, 'H', 'N' },
{ QLocale::Language::Croatian, QLocale::Country::Croatia, 'H', 'R' },
{ QLocale::Language::Haitian, QLocale::Country::Haiti, 'H', 'T' },
{ QLocale::Language::Hungarian, QLocale::Country::Hungary, 'H', 'U' },
{ QLocale::Language::Spanish, QLocale::Country::CanaryIslands, 'I', 'C' },
{ QLocale::Language::Indonesian, QLocale::Country::Indonesia, 'I', 'D' },
{ QLocale::Language::Hebrew, QLocale::Country::Israel, 'I', 'L' },
{ QLocale::Language::AnyLanguage, QLocale::Country::India, 'I', 'N' },
{ QLocale::Language::Arabic, QLocale::Country::Iraq, 'I', 'Q' },
{ QLocale::Language::Persian, QLocale::Country::Iran, 'I', 'R' },
{ QLocale::Language::Icelandic, QLocale::Country::Iceland, 'I', 'S' },
{ QLocale::Language::Italian, QLocale::Country::Italy, 'I', 'T' },
{ QLocale::Language::Arabic, QLocale::Country::Jordan, 'J', 'O' },
{ QLocale::Language::Japanese, QLocale::Country::Japan, 'J', 'P' },
{ QLocale::Language::Swahili, QLocale::Country::Kenya, 'K', 'E' },
{ QLocale::Language::Kirghiz, QLocale::Country::Kyrgyzstan, 'K', 'G' },
{ QLocale::Language::Khmer, QLocale::Country::Cambodia, 'K', 'H' },
{ QLocale::Language::Arabic, QLocale::Country::Comoros, 'K', 'M' },
{ QLocale::Language::Korean, QLocale::Country::NorthKorea, 'K', 'P' },
{ QLocale::Language::Korean, QLocale::Country::SouthKorea, 'K', 'R' },
{ QLocale::Language::Arabic, QLocale::Country::Kuwait, 'K', 'W' },
{ QLocale::Language::Russian, QLocale::Country::Kazakhstan, 'K', 'Z' },
{ QLocale::Language::Lao, QLocale::Country::Laos, 'L', 'A' },
{ QLocale::Language::Arabic, QLocale::Country::Lebanon, 'L', 'B' },
{ QLocale::Language::German, QLocale::Country::Liechtenstein, 'L', 'I' },
{ QLocale::Language::Sinhala, QLocale::Country::SriLanka, 'L', 'K' },
{ QLocale::Language::SouthernSotho, QLocale::Country::Lesotho, 'L', 'S' },
{ QLocale::Language::Lithuanian, QLocale::Country::Lithuania, 'L', 'T' },
{ QLocale::Language::French, QLocale::Country::Luxembourg, 'L', 'U' },
{ QLocale::Language::Latvian, QLocale::Country::Latvia, 'L', 'V' },
{ QLocale::Language::Arabic, QLocale::Country::Libya, 'L', 'Y' },
{ QLocale::Language::Arabic, QLocale::Country::Morocco, 'M', 'A' },
{ QLocale::Language::French, QLocale::Country::Monaco, 'M', 'C' },
{ QLocale::Language::Romanian, QLocale::Country::Moldova, 'M', 'D' },
{ QLocale::Language::Serbian, QLocale::Country::Montenegro, 'M', 'E' },
{ QLocale::Language::French, QLocale::Country::SaintMartin, 'M', 'F' },
{ QLocale::Language::Malagasy, QLocale::Country::Madagascar, 'M', 'G' },
{ QLocale::Language::Macedonian, QLocale::Country::Macedonia, 'M', 'K' },
{ QLocale::Language::Bambara, QLocale::Country::Mali, 'M', 'L' },
{ QLocale::Language::Burmese, QLocale::Country::Myanmar, 'M', 'M' },
{ QLocale::Language::Mongolian, QLocale::Country::Mongolia, 'M', 'N' },
{ QLocale::Language::Chinese, QLocale::Country::Macau, 'M', 'O' },
{ QLocale::Language::French, QLocale::Country::Martinique, 'M', 'Q' },
{ QLocale::Language::Arabic, QLocale::Country::Mauritania, 'M', 'R' },
{ QLocale::Language::Maltese, QLocale::Country::Malta, 'M', 'T' },
{ QLocale::Language::Morisyen, QLocale::Country::Mauritius, 'M', 'U' },
{ QLocale::Language::Divehi, QLocale::Country::Maldives, 'M', 'V' },
{ QLocale::Language::Spanish, QLocale::Country::Mexico, 'M', 'X' },
{ QLocale::Language::Malay, QLocale::Country::Malaysia, 'M', 'Y' },
{ QLocale::Language::Portuguese, QLocale::Country::Mozambique, 'M', 'Z' },
{ QLocale::Language::Afrikaans, QLocale::Country::Namibia, 'N', 'A' },
{ QLocale::Language::French, QLocale::Country::NewCaledonia, 'N', 'C' },
{ QLocale::Language::Hausa, QLocale::Country::Niger, 'N', 'E' },
{ QLocale::Language::Spanish, QLocale::Country::Nicaragua, 'N', 'I' },
{ QLocale::Language::Dutch, QLocale::Country::Netherlands, 'N', 'L' },
{ QLocale::Language::NorwegianBokmal, QLocale::Country::Norway, 'N', 'O' },
{ QLocale::Language::Nepali, QLocale::Country::Nepal, 'N', 'P' },
{ QLocale::Language::Arabic, QLocale::Country::Oman, 'O', 'M' },
{ QLocale::Language::Spanish, QLocale::Country::Panama, 'P', 'A' },
{ QLocale::Language::Spanish, QLocale::Country::Peru, 'P', 'E' },
{ QLocale::Language::French, QLocale::Country::FrenchPolynesia, 'P', 'F' },
{ QLocale::Language::TokPisin, QLocale::Country::PapuaNewGuinea, 'P', 'G' },
{ QLocale::Language::Filipino, QLocale::Country::Philippines, 'P', 'H' },
{ QLocale::Language::Urdu, QLocale::Country::Pakistan, 'P', 'K' },
{ QLocale::Language::Polish, QLocale::Country::Poland, 'P', 'L' },
{ QLocale::Language::French, QLocale::Country::SaintPierreAndMiquelon, 'P', 'M' },
{ QLocale::Language::Spanish, QLocale::Country::PuertoRico, 'P', 'R' },
{ QLocale::Language::Arabic, QLocale::Country::PalestinianTerritories, 'P', 'S' },
{ QLocale::Language::Portuguese, QLocale::Country::Portugal, 'P', 'T' },
{ QLocale::Language::Palauan, QLocale::Country::Palau, 'P', 'W' },
{ QLocale::Language::Guarani, QLocale::Country::Paraguay, 'P', 'Y' },
{ QLocale::Language::Arabic, QLocale::Country::Qatar, 'Q', 'A' },
{ QLocale::Language::English, QLocale::Country::OutlyingOceania, 'Q', 'O' },
{ QLocale::Language::French, QLocale::Country::Reunion, 'R', 'E' },
{ QLocale::Language::Romanian, QLocale::Country::Romania, 'R', 'O' },
{ QLocale::Language::Serbian, QLocale::Country::Serbia, 'R', 'S' },
{ QLocale::Language::Russian, QLocale::Country::Russia, 'R', 'U' },
{ QLocale::Language::Kinyarwanda, QLocale::Country::Rwanda, 'R', 'W' },
{ QLocale::Language::Arabic, QLocale::Country::SaudiArabia, 'S', 'A' },
{ QLocale::Language::French, QLocale::Country::Seychelles, 'S', 'C' },
{ QLocale::Language::Arabic, QLocale::Country::Sudan, 'S', 'D' },
{ QLocale::Language::Swedish, QLocale::Country::Sweden, 'S', 'E' },
{ QLocale::Language::Slovenian, QLocale::Country::Slovenia, 'S', 'I' },
{ QLocale::Language::NorwegianBokmal, QLocale::Country::SvalbardAndJanMayenIslands, 'S', 'J' },
{ QLocale::Language::Slovak, QLocale::Country::Slovakia, 'S', 'K' },
{ QLocale::Language::Italian, QLocale::Country::SanMarino, 'S', 'M' },
{ QLocale::Language::French, QLocale::Country::Senegal, 'S', 'N' },
{ QLocale::Language::Somali, QLocale::Country::Somalia, 'S', 'O' },
{ QLocale::Language::Dutch, QLocale::Country::Suriname, 'S', 'R' },
{ QLocale::Language::Portuguese, QLocale::Country::SaoTomeAndPrincipe, 'S', 'T' },
{ QLocale::Language::Spanish, QLocale::Country::ElSalvador, 'S', 'V' },
{ QLocale::Language::Arabic, QLocale::Country::Syria, 'S', 'Y' },
{ QLocale::Language::French, QLocale::Country::Chad, 'T', 'D' },
{ QLocale::Language::French, QLocale::Country::FrenchSouthernTerritories, 'T', 'F' },
{ QLocale::Language::French, QLocale::Country::Togo, 'T', 'G' },
{ QLocale::Language::Thai, QLocale::Country::Thailand, 'T', 'H' },
{ QLocale::Language::Tajik, QLocale::Country::Tajikistan, 'T', 'J' },
{ QLocale::Language::TokelauLanguage, QLocale::Country::TokelauCountry, 'T', 'K' },
{ QLocale::Language::Portuguese, QLocale::Country::EastTimor, 'T', 'L' },
{ QLocale::Language::Turkmen, QLocale::Country::Turkmenistan, 'T', 'M' },
{ QLocale::Language::Arabic, QLocale::Country::Tunisia, 'T', 'N' },
{ QLocale::Language::Tongan, QLocale::Country::Tonga, 'T', 'O' },
{ QLocale::Language::Turkish, QLocale::Country::Turkey, 'T', 'R' },
{ QLocale::Language::TuvaluLanguage, QLocale::Country::TuvaluCountry, 'T', 'V' },
{ QLocale::Language::Chinese, QLocale::Country::Taiwan, 'T', 'W' },
{ QLocale::Language::Swahili, QLocale::Country::Tanzania, 'T', 'Z' },
{ QLocale::Language::Ukrainian, QLocale::Country::Ukraine, 'U', 'A' },
{ QLocale::Language::Swahili, QLocale::Country::Uganda, 'U', 'G' },
{ QLocale::Language::Spanish, QLocale::Country::Uruguay, 'U', 'Y' },
{ QLocale::Language::Uzbek, QLocale::Country::Uzbekistan, 'U', 'Z' },
{ QLocale::Language::Italian, QLocale::Country::VaticanCityState, 'V', 'A' },
{ QLocale::Language::Spanish, QLocale::Country::Venezuela, 'V', 'E' },
{ QLocale::Language::Vietnamese, QLocale::Country::Vietnam, 'V', 'N' },
{ QLocale::Language::Bislama, QLocale::Country::Vanuatu, 'V', 'U' },
{ QLocale::Language::French, QLocale::Country::WallisAndFutunaIslands, 'W', 'F' },
{ QLocale::Language::Samoan, QLocale::Country::Samoa, 'W', 'S' },
{ QLocale::Language::Albanian, QLocale::Country::Kosovo, 'X', 'K' },
{ QLocale::Language::Arabic, QLocale::Country::Yemen, 'Y', 'E' },
{ QLocale::Language::French, QLocale::Country::Mayotte, 'Y', 'T' },
{ QLocale::Language::Shona, QLocale::Country::Zimbabwe, 'Z', 'W' },
{ QLocale::Language::AnyLanguage, QLocale::Country::AnyCountry, 0, 0 },
};

static_assert( (sizeof(country_data_table) / sizeof(CountryData)) == country_data_size, "Table size mismatch for CountryData" );


// END Generated from CLDR data
