<?xml version="1.0" ?>
<!-- SPDX-FileCopyrightText: no
     SPDX-License-Identifier: CC0-1.0
-->
<!DOCTYPE TS><TS version="2.1" language="en_US">
<context>
    <name>QObject</name>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="30"/>
        <source>Africa</source>
        <comment>tz_regions</comment>
        <translation>Afrika</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="31"/>
        <source>America</source>
        <comment>tz_regions</comment>
        <translation>Amerika</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="32"/>
        <source>Antarctica</source>
        <comment>tz_regions</comment>
        <translation>Antarktis</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="34"/>
        <source>Asia</source>
        <comment>tz_regions</comment>
        <translation>Asien</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="36"/>
        <source>Australia</source>
        <comment>tz_regions</comment>
        <translation>Australien</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="37"/>
        <source>Europe</source>
        <comment>tz_regions</comment>
        <translation>Europa</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="38"/>
        <source>Indian</source>
        <comment>tz_regions</comment>
        <translation>Indien</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="68"/>
        <source>Aqtobe</source>
        <comment>tz_names</comment>
        <translation>Aqtöbe</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="87"/>
        <source>Athens</source>
        <comment>tz_names</comment>
        <translation>Aten</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="91"/>
        <source>Azores</source>
        <comment>tz_names</comment>
        <translation>Azorerna</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="253"/>
        <source>Kiev</source>
        <comment>tz_names</comment>
        <translation>Kyiv</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="326"/>
        <source>Moscow</source>
        <comment>tz_names</comment>
        <translation>Moskva</translation>
    </message>
</context>
</TS>
