<?xml version="1.0" encoding="utf-8"?>
<!-- SPDX-FileCopyrightText: no
     SPDX-License-Identifier: CC0-1.0
-->
<!DOCTYPE TS>
<TS version="2.1" language="nl">
<context>
    <name>QObject</name>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="30"/>
        <source>Africa</source>
        <comment>tz_regions</comment>
        <translation>Afrika</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="31"/>
        <source>America</source>
        <comment>tz_regions</comment>
        <translation>Amerika</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="32"/>
        <source>Antarctica</source>
        <comment>tz_regions</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="33"/>
        <source>Arctic</source>
        <comment>tz_regions</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="34"/>
        <source>Asia</source>
        <comment>tz_regions</comment>
        <translation>Azië</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="35"/>
        <source>Atlantic</source>
        <comment>tz_regions</comment>
        <translation>Atlantisch</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="36"/>
        <source>Australia</source>
        <comment>tz_regions</comment>
        <translation>Australië</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="37"/>
        <source>Europe</source>
        <comment>tz_regions</comment>
        <translation>Europa</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="38"/>
        <source>Indian</source>
        <comment>tz_regions</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="39"/>
        <source>Pacific</source>
        <comment>tz_regions</comment>
        <translation>Stille Oceaan</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="50"/>
        <source>Abidjan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="51"/>
        <source>Accra</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="52"/>
        <source>Adak</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="53"/>
        <source>Addis Ababa</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="54"/>
        <source>Adelaide</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="55"/>
        <source>Aden</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="56"/>
        <source>Algiers</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="57"/>
        <source>Almaty</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="58"/>
        <source>Amman</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="59"/>
        <source>Amsterdam</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="60"/>
        <source>Anadyr</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="61"/>
        <source>Anchorage</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="62"/>
        <source>Andorra</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="63"/>
        <source>Anguilla</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="64"/>
        <source>Antananarivo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="65"/>
        <source>Antigua</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="66"/>
        <source>Apia</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="67"/>
        <source>Aqtau</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="68"/>
        <source>Aqtobe</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="69"/>
        <source>Araguaina</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="70"/>
        <source>Argentina/Buenos Aires</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="71"/>
        <source>Argentina/Catamarca</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="72"/>
        <source>Argentina/Cordoba</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="73"/>
        <source>Argentina/Jujuy</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="74"/>
        <source>Argentina/La Rioja</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="75"/>
        <source>Argentina/Mendoza</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="76"/>
        <source>Argentina/Rio Gallegos</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="77"/>
        <source>Argentina/Salta</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="78"/>
        <source>Argentina/San Juan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="79"/>
        <source>Argentina/San Luis</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="80"/>
        <source>Argentina/Tucuman</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="81"/>
        <source>Argentina/Ushuaia</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="82"/>
        <source>Aruba</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="83"/>
        <source>Ashgabat</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="84"/>
        <source>Asmara</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="85"/>
        <source>Astrakhan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="86"/>
        <source>Asuncion</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="87"/>
        <source>Athens</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="88"/>
        <source>Atikokan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="89"/>
        <source>Atyrau</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="90"/>
        <source>Auckland</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="91"/>
        <source>Azores</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="92"/>
        <source>Baghdad</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="93"/>
        <source>Bahia</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="94"/>
        <source>Bahia Banderas</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="95"/>
        <source>Bahrain</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="96"/>
        <source>Baku</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="97"/>
        <source>Bamako</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="98"/>
        <source>Bangkok</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="99"/>
        <source>Bangui</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="100"/>
        <source>Banjul</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="101"/>
        <source>Barbados</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="102"/>
        <source>Barnaul</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="103"/>
        <source>Beirut</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="104"/>
        <source>Belem</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="105"/>
        <source>Belgrade</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="106"/>
        <source>Belize</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="107"/>
        <source>Berlin</source>
        <comment>tz_names</comment>
        <translation>Berlijn</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="108"/>
        <source>Bermuda</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="109"/>
        <source>Bishkek</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="110"/>
        <source>Bissau</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="111"/>
        <source>Blanc-Sablon</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="112"/>
        <source>Blantyre</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="113"/>
        <source>Boa Vista</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="114"/>
        <source>Bogota</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="115"/>
        <source>Boise</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="116"/>
        <source>Bougainville</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="117"/>
        <source>Bratislava</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="118"/>
        <source>Brazzaville</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="119"/>
        <source>Brisbane</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="120"/>
        <source>Broken Hill</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="121"/>
        <source>Brunei</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="122"/>
        <source>Brussels</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="123"/>
        <source>Bucharest</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="124"/>
        <source>Budapest</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="125"/>
        <source>Bujumbura</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="126"/>
        <source>Busingen</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="127"/>
        <source>Cairo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="128"/>
        <source>Cambridge Bay</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="129"/>
        <source>Campo Grande</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="130"/>
        <source>Canary</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="131"/>
        <source>Cancun</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="132"/>
        <source>Cape Verde</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="133"/>
        <source>Caracas</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="134"/>
        <source>Casablanca</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="135"/>
        <source>Casey</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="136"/>
        <source>Cayenne</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="137"/>
        <source>Cayman</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="138"/>
        <source>Ceuta</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="139"/>
        <source>Chagos</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="140"/>
        <source>Chatham</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="141"/>
        <source>Chicago</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="142"/>
        <source>Chihuahua</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="143"/>
        <source>Chisinau</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="144"/>
        <source>Chita</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="145"/>
        <source>Choibalsan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="146"/>
        <source>Christmas</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="147"/>
        <source>Chuuk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="148"/>
        <source>Cocos</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="149"/>
        <source>Colombo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="150"/>
        <source>Comoro</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="151"/>
        <source>Conakry</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="152"/>
        <source>Copenhagen</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="153"/>
        <source>Costa Rica</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="154"/>
        <source>Creston</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="155"/>
        <source>Cuiaba</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="156"/>
        <source>Curacao</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="157"/>
        <source>Currie</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="158"/>
        <source>Dakar</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="159"/>
        <source>Damascus</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="160"/>
        <source>Danmarkshavn</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="161"/>
        <source>Dar es Salaam</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="162"/>
        <source>Darwin</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="163"/>
        <source>Davis</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="164"/>
        <source>Dawson</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="165"/>
        <source>Dawson Creek</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="166"/>
        <source>Denver</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="167"/>
        <source>Detroit</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="168"/>
        <source>Dhaka</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="169"/>
        <source>Dili</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="170"/>
        <source>Djibouti</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="171"/>
        <source>Dominica</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="172"/>
        <source>Douala</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="173"/>
        <source>Dubai</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="174"/>
        <source>Dublin</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="175"/>
        <source>DumontDUrville</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="176"/>
        <source>Dushanbe</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="177"/>
        <source>Easter</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="178"/>
        <source>Edmonton</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="179"/>
        <source>Efate</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="180"/>
        <source>Eirunepe</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="181"/>
        <source>El Aaiun</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="182"/>
        <source>El Salvador</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="183"/>
        <source>Enderbury</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="184"/>
        <source>Eucla</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="185"/>
        <source>Fakaofo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="186"/>
        <source>Famagusta</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="187"/>
        <source>Faroe</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="188"/>
        <source>Fiji</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="189"/>
        <source>Fort Nelson</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="190"/>
        <source>Fortaleza</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="191"/>
        <source>Freetown</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="192"/>
        <source>Funafuti</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="193"/>
        <source>Gaborone</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="194"/>
        <source>Galapagos</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="195"/>
        <source>Gambier</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="196"/>
        <source>Gaza</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="197"/>
        <source>Gibraltar</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="198"/>
        <source>Glace Bay</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="199"/>
        <source>Godthab</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="200"/>
        <source>Goose Bay</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="201"/>
        <source>Grand Turk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="202"/>
        <source>Grenada</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="203"/>
        <source>Guadalcanal</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="204"/>
        <source>Guadeloupe</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="205"/>
        <source>Guam</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="206"/>
        <source>Guatemala</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="207"/>
        <source>Guayaquil</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="208"/>
        <source>Guernsey</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="209"/>
        <source>Guyana</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="210"/>
        <source>Halifax</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="211"/>
        <source>Harare</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="212"/>
        <source>Havana</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="213"/>
        <source>Hebron</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="214"/>
        <source>Helsinki</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="215"/>
        <source>Hermosillo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="216"/>
        <source>Ho Chi Minh</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="217"/>
        <source>Hobart</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="218"/>
        <source>Hong Kong</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="219"/>
        <source>Honolulu</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="220"/>
        <source>Hovd</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="221"/>
        <source>Indiana/Indianapolis</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="222"/>
        <source>Indiana/Knox</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="223"/>
        <source>Indiana/Marengo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="224"/>
        <source>Indiana/Petersburg</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="225"/>
        <source>Indiana/Tell City</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="226"/>
        <source>Indiana/Vevay</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="227"/>
        <source>Indiana/Vincennes</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="228"/>
        <source>Indiana/Winamac</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="229"/>
        <source>Inuvik</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="230"/>
        <source>Iqaluit</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="231"/>
        <source>Irkutsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="232"/>
        <source>Isle of Man</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="233"/>
        <source>Istanbul</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="234"/>
        <source>Jakarta</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="235"/>
        <source>Jamaica</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="236"/>
        <source>Jayapura</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="237"/>
        <source>Jersey</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="238"/>
        <source>Jerusalem</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="239"/>
        <source>Johannesburg</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="240"/>
        <source>Juba</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="241"/>
        <source>Juneau</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="242"/>
        <source>Kabul</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="243"/>
        <source>Kaliningrad</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="244"/>
        <source>Kamchatka</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="245"/>
        <source>Kampala</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="246"/>
        <source>Karachi</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="247"/>
        <source>Kathmandu</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="248"/>
        <source>Kentucky/Louisville</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="249"/>
        <source>Kentucky/Monticello</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="250"/>
        <source>Kerguelen</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="251"/>
        <source>Khandyga</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="252"/>
        <source>Khartoum</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="253"/>
        <source>Kiev</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="254"/>
        <source>Kigali</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="255"/>
        <source>Kinshasa</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="256"/>
        <source>Kiritimati</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="257"/>
        <source>Kirov</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="258"/>
        <source>Kolkata</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="259"/>
        <source>Kosrae</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="260"/>
        <source>Kralendijk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="261"/>
        <source>Krasnoyarsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="262"/>
        <source>Kuala Lumpur</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="263"/>
        <source>Kuching</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="264"/>
        <source>Kuwait</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="265"/>
        <source>Kwajalein</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="266"/>
        <source>La Paz</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="267"/>
        <source>Lagos</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="268"/>
        <source>Libreville</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="269"/>
        <source>Lima</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="270"/>
        <source>Lindeman</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="271"/>
        <source>Lisbon</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="272"/>
        <source>Ljubljana</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="273"/>
        <source>Lome</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="274"/>
        <source>London</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="275"/>
        <source>Longyearbyen</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="276"/>
        <source>Lord Howe</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="277"/>
        <source>Los Angeles</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="278"/>
        <source>Lower Princes</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="279"/>
        <source>Luanda</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="280"/>
        <source>Lubumbashi</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="281"/>
        <source>Lusaka</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="282"/>
        <source>Luxembourg</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="283"/>
        <source>Macau</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="284"/>
        <source>Maceio</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="285"/>
        <source>Macquarie</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="286"/>
        <source>Madeira</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="287"/>
        <source>Madrid</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="288"/>
        <source>Magadan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="289"/>
        <source>Mahe</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="290"/>
        <source>Majuro</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="291"/>
        <source>Makassar</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="292"/>
        <source>Malabo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="293"/>
        <source>Maldives</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="294"/>
        <source>Malta</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="295"/>
        <source>Managua</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="296"/>
        <source>Manaus</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="297"/>
        <source>Manila</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="298"/>
        <source>Maputo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="299"/>
        <source>Mariehamn</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="300"/>
        <source>Marigot</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="301"/>
        <source>Marquesas</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="302"/>
        <source>Martinique</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="303"/>
        <source>Maseru</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="304"/>
        <source>Matamoros</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="305"/>
        <source>Mauritius</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="306"/>
        <source>Mawson</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="307"/>
        <source>Mayotte</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="308"/>
        <source>Mazatlan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="309"/>
        <source>Mbabane</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="310"/>
        <source>McMurdo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="311"/>
        <source>Melbourne</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="312"/>
        <source>Menominee</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="313"/>
        <source>Merida</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="314"/>
        <source>Metlakatla</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="315"/>
        <source>Mexico City</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="316"/>
        <source>Midway</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="317"/>
        <source>Minsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="318"/>
        <source>Miquelon</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="319"/>
        <source>Mogadishu</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="320"/>
        <source>Monaco</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="321"/>
        <source>Moncton</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="322"/>
        <source>Monrovia</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="323"/>
        <source>Monterrey</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="324"/>
        <source>Montevideo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="325"/>
        <source>Montserrat</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="326"/>
        <source>Moscow</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="327"/>
        <source>Muscat</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="328"/>
        <source>Nairobi</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="329"/>
        <source>Nassau</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="330"/>
        <source>Nauru</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="331"/>
        <source>Ndjamena</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="332"/>
        <source>New York</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="333"/>
        <source>Niamey</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="334"/>
        <source>Nicosia</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="335"/>
        <source>Nipigon</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="336"/>
        <source>Niue</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="337"/>
        <source>Nome</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="338"/>
        <source>Norfolk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="339"/>
        <source>Noronha</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="340"/>
        <source>North Dakota/Beulah</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="341"/>
        <source>North Dakota/Center</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="342"/>
        <source>North Dakota/New Salem</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="343"/>
        <source>Nouakchott</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="344"/>
        <source>Noumea</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="345"/>
        <source>Novokuznetsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="346"/>
        <source>Novosibirsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="347"/>
        <source>Ojinaga</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="348"/>
        <source>Omsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="349"/>
        <source>Oral</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="350"/>
        <source>Oslo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="351"/>
        <source>Ouagadougou</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="352"/>
        <source>Pago Pago</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="353"/>
        <source>Palau</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="354"/>
        <source>Palmer</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="355"/>
        <source>Panama</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="356"/>
        <source>Pangnirtung</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="357"/>
        <source>Paramaribo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="358"/>
        <source>Paris</source>
        <comment>tz_names</comment>
        <translation>Parijs</translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="359"/>
        <source>Perth</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="360"/>
        <source>Phnom Penh</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="361"/>
        <source>Phoenix</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="362"/>
        <source>Pitcairn</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="363"/>
        <source>Podgorica</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="364"/>
        <source>Pohnpei</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="365"/>
        <source>Pontianak</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="366"/>
        <source>Port Moresby</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="367"/>
        <source>Port of Spain</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="368"/>
        <source>Port-au-Prince</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="369"/>
        <source>Porto Velho</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="370"/>
        <source>Porto-Novo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="371"/>
        <source>Prague</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="372"/>
        <source>Puerto Rico</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="373"/>
        <source>Punta Arenas</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="374"/>
        <source>Pyongyang</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="375"/>
        <source>Qatar</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="376"/>
        <source>Qostanay</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="377"/>
        <source>Qyzylorda</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="378"/>
        <source>Rainy River</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="379"/>
        <source>Rankin Inlet</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="380"/>
        <source>Rarotonga</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="381"/>
        <source>Recife</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="382"/>
        <source>Regina</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="383"/>
        <source>Resolute</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="384"/>
        <source>Reunion</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="385"/>
        <source>Reykjavik</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="386"/>
        <source>Riga</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="387"/>
        <source>Rio Branco</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="388"/>
        <source>Riyadh</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="389"/>
        <source>Rome</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="390"/>
        <source>Rothera</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="391"/>
        <source>Saipan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="392"/>
        <source>Sakhalin</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="393"/>
        <source>Samara</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="394"/>
        <source>Samarkand</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="395"/>
        <source>San Marino</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="396"/>
        <source>Santarem</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="397"/>
        <source>Santiago</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="398"/>
        <source>Santo Domingo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="399"/>
        <source>Sao Paulo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="400"/>
        <source>Sao Tome</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="401"/>
        <source>Sarajevo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="402"/>
        <source>Saratov</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="403"/>
        <source>Scoresbysund</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="404"/>
        <source>Seoul</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="405"/>
        <source>Shanghai</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="406"/>
        <source>Simferopol</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="407"/>
        <source>Singapore</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="408"/>
        <source>Sitka</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="409"/>
        <source>Skopje</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="410"/>
        <source>Sofia</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="411"/>
        <source>South Georgia</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="412"/>
        <source>Srednekolymsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="413"/>
        <source>St Barthelemy</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="414"/>
        <source>St Helena</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="415"/>
        <source>St Johns</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="416"/>
        <source>St Kitts</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="417"/>
        <source>St Lucia</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="418"/>
        <source>St Thomas</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="419"/>
        <source>St Vincent</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="420"/>
        <source>Stanley</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="421"/>
        <source>Stockholm</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="422"/>
        <source>Swift Current</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="423"/>
        <source>Sydney</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="424"/>
        <source>Syowa</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="425"/>
        <source>Tahiti</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="426"/>
        <source>Taipei</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="427"/>
        <source>Tallinn</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="428"/>
        <source>Tarawa</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="429"/>
        <source>Tashkent</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="430"/>
        <source>Tbilisi</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="431"/>
        <source>Tegucigalpa</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="432"/>
        <source>Tehran</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="433"/>
        <source>Thimphu</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="434"/>
        <source>Thule</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="435"/>
        <source>Thunder Bay</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="436"/>
        <source>Tijuana</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="437"/>
        <source>Tirane</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="438"/>
        <source>Tokyo</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="439"/>
        <source>Tomsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="440"/>
        <source>Tongatapu</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="441"/>
        <source>Toronto</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="442"/>
        <source>Tortola</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="443"/>
        <source>Tripoli</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="444"/>
        <source>Troll</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="445"/>
        <source>Tunis</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="446"/>
        <source>Ulaanbaatar</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="447"/>
        <source>Ulyanovsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="448"/>
        <source>Urumqi</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="449"/>
        <source>Ust-Nera</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="450"/>
        <source>Uzhgorod</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="451"/>
        <source>Vaduz</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="452"/>
        <source>Vancouver</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="453"/>
        <source>Vatican</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="454"/>
        <source>Vienna</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="455"/>
        <source>Vientiane</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="456"/>
        <source>Vilnius</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="457"/>
        <source>Vladivostok</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="458"/>
        <source>Volgograd</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="459"/>
        <source>Vostok</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="460"/>
        <source>Wake</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="461"/>
        <source>Wallis</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="462"/>
        <source>Warsaw</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="463"/>
        <source>Whitehorse</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="464"/>
        <source>Windhoek</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="465"/>
        <source>Winnipeg</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="466"/>
        <source>Yakutat</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="467"/>
        <source>Yakutsk</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="468"/>
        <source>Yangon</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="469"/>
        <source>Yekaterinburg</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="470"/>
        <source>Yellowknife</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="471"/>
        <source>Yerevan</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="472"/>
        <source>Zagreb</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="473"/>
        <source>Zaporozhye</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
    <message>
        <location filename="../src/libcalamares/locale/ZoneData_p.cxxtr" line="474"/>
        <source>Zurich</source>
        <comment>tz_names</comment>
        <translation type="unfinished"></translation>
    </message>
</context>
</TS>
